function [out]=funcbaskpopMod(data)
% Antoine-Besse April 2006
% Computation of the solution of the free Schroedinger equation
% with Absorbing Boundary conditions computed by
% Baskakov-Popov scheme.

% data is a cell
%  |
%  |--- data.a
%  |--- data.b
%  |--- data.sigma
%  |--- data.xc
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.initDatum
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
% Setting default values
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
savenb=data.savenb;
a=data.a;
b=data.b;
sigma=data.sigma;
xc=data.xc;
x0=data.x0; 
x1=data.x1;
N=data.N;
dx=(x1-x0)/(N+1);
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=tfinal/dt;
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end
% Wave number
k0=data.k0;

x = (x0:dx:x1)';
gridx = x;
% Initial datum
%u1=initialDatum(x,k0);
u=eval(data.initDatum);  
try
  V = potential_func(data.xc);
  a = data.a;
  b = data.b;
  xc = data.xc;
  k0 = data.k0;
  sigma = data.sigma;
  t = 0;
  u = eval(data.exactCpot); 
catch
  warning('Using initDatum') 
end

%-----------------------------------------------------------
% Matrices assembly
%-----------------------------------------------------------
E = exp(-i*pi/4);
dfract = 2*E/sqrt(pi*dt);

MT = dx*speye(N+2,N+2);

MB = -1/dx*spdiags([ones(N+2,1),-2*ones(N+2,1),ones(N+2,1)],-1:1,N+2,N+2);
MB(1,1) = 2/dx;
MB(1,2) = -2/dx;
MB(end,end) = 2/dx;
MB(end,end-1) = -2/dx;

MC = sparse(N+2,N+2);
MC(1,1) = dfract;
MC(end,end) = dfract;

MD = dx*spdiags(potential_func(x),0,N+2,N+2);

A =  a*i*MT/dt-b*MB/2-b*sqrt(a/b)*MC-MD/2;
B =  a*i*MT/dt+b*MB/2+b*sqrt(a/b)*MC+MD/2;

Vl  = potential_func(x0)/a; % Potential value on xl
Vr  = potential_func(x1)/a; % Potential value on xr
Vlt = zeros(1,niter);
Vrt = zeros(1,niter);

%
% Computation of convolution terms
%
for j=1:niter+1,
  gammak(j)=2/((sqrt(j+1)+sqrt(j))*(sqrt(j)+sqrt(j-1))*(sqrt(j+1)+sqrt(j-1)));
end;

SM_old = zeros(N+2,1);
SM_new = SM_old;
% LU factorization of matrix A
%[L,U]=lu(A);
[LA,UA,pA,qA] = lu(A,'vector');
sav=1;
% pl : used to store |u^n|
pl=zeros(niter/savenb,N+2);
pl2=pl;
pl(1,:)=u.';
tl=zeros(niter/savenb,1);
tl(1)=0;

%--------------------------------------------------------------

MaxV=max(potential_func(x));
if (MaxV==0)
  MaxV=1;
end;
% Iterations
for k=1:niter,
% Resolution of the linear system
%  y=L\(B*u-b*sqrt(a/b)*(SM_old+SM_new));
%  u=U\y;
  rhs = B*u-b*sqrt(a/b)*(SM_old+SM_new);
  u(qA,:)= UA\(LA\rhs(pA,:));
  
% Storage of the boundary terms
  Vlt(k) = u(1)*exp(i*Vl*k*dt);
  Vrt(k) = u(N+2)*exp(i*Vr*k*dt);
% Computation of the futur R.H.S by convolution
  gam     = gammak(1:k);
  SM_old = SM_new;
  SM_new(1)   = dfract*exp(-i*Vl*(k+1)*dt)*fliplr(Vlt(1:k))*gam';
  SM_new(N+2) = dfract*exp(-i*Vr*(k+1)*dt)*fliplr(Vrt(1:k))*gam';

% Display
  if rem(k,savenb)==0
    sav=sav+1;
    pl(sav,:)=u.';
    tl(sav)=k*dt;
    if   data.PlotFlag
      plot(x,abs(u1),x,pot/MaxV,'r:','parent',data.axe1);axis(data.axe1,[x0 x1 0 1.1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
	pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
	axis(data.axe2,[x0 x1 0 tfinal]);
	h=colorbar('peer',data.axe2);
	set(h,'fontsize',7);
      end
      %K=[K;k*dt];ERR=[ERR;l2simpson(dx,abs(u_eccard))/l2ex];      
    end    
  end
end

%  figure(2);
%  mesh(x,tl,pl);view(160,60);
if data.PlotFlag
  pl2(1:sav,:)=log10(pl(1:sav,:));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
           'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end
% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end  