function [out]=funcfem(data)
% Antoine-Besse July 2006
% Computation of the solution of the free Schroedinger equation
% by Crank-Nicolson scheme with a linear finite element method
% with Absorbing Boundary conditions : discretization of Antoine Besse
% with Dirichlet-Neuman formulation.

% data is a cell
%  |
%  |--- data.a
%  |--- data.b
%  |--- data.sigma
%  |--- data.xc
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.initDatum
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
a=data.a;
b=data.b;
sigma=data.sigma;
xc=data.xc;
x0=data.x0; 
x1=data.x1;
N=data.N;
dx=(x1-x0)/(N+1);
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=tfinal/dt;
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end
% Wave number
k0=data.k0;

x=(x0:dx:x1)';
gridx = x;
xnorm=(x0:dx/2:x1)';

% Initial datum
%u1=initialDatum(x,k0);
u1=eval(data.initDatum);  
u2=u1;
% E=exp(I*Pi/4)
E=(sqrt(2)+i*sqrt(2))/2;

% -------------------------------------------------------------------------
%
% The system to solve for any test functions v on [x_l,x_r]
%
%   i                 1                    1
% ---- <u^(n+1),v> - --- <dxu^(n+1),dxv>+ --- ([dxu^(n+1) v](x_r)+[dxu^(n+1) v](x_l))=
%  dt                 2                    2 
%
%   i                 1                    1
% ---- <u^(n),v>   + --- <dxu^(n),dxv>  - --- ([dxu^(n) v](x_r)-[dxu^(n) v](x_l))
%  dt                 2                    2
%
% which reads
%
%   i                                       i
% ---- A u^(n+1) - B u^(n+1) +C u^(n+1) = ---- A u^(n) + B u^(n) - C u^(n)
%  dt                                      dt
%
% -------------------------------------------------------------------------
A=massMatrix(N,dx);
B=stiffnessMatrix(N,dx);
boundaryTerm=-sqrt(2/dt)/E;
C=boundaryMatrix(N,boundaryTerm);
D=massMatrixPot(x0,x1,N,dx);

% Assembly : matrices of the system
M1=a*i*A/dt-b*B/2+b*sqrt(a/b)*C/2-D/2;
M2=a*i*A/dt+b*B/2-b*sqrt(a/b)*C/2+D/2;
%
% LU factorization of M1 matrix
%
[L,U]=lu(M1);
%
% Computation of convolution terms
%
alpha=convolTerms(niter+1,E,dt);
%
% Remember the boundary values
Vl=potential_func(x0); % Potential value on xl
Vr=potential_func(x1); % Potential value on xr
Vlt=[u2(1)*exp(i*Vl*0) ];
Vrt=[u2(N+2)*exp(i*Vr*0) ];
SM_old=zeros(N+2,1);
SM_new=SM_old;


sav=1;
pl=zeros(niter/25,N+2);
pl(1,:)=abs(u1)';   % Vector to store the evolution of u
% Useful to plot the log-error
tl=zeros(niter/25,1);
tl(1)=0;
% $$$ K=[0];ERR=[0];tmp=0;
% $$$ unorm=zeros(N+2+N+1,1);
% $$$ unorm(2:2:2*N+3)=(u1(1:N+1)+u1(2:N+2))/2;
% $$$ unorm(1:2:2*N+3)=u1;
% $$$ l2=l2simpson(dx,abs(unorm));
% $$$ L2NORM=[l2]; % Vector to store the evolution of the L2 norm of the approximate solution
% $$$ L2NORMEX=[l2]; % Vector to store the evolution of the L2 norm of the exact solution

MaxV=max(potential_func(x));
if (MaxV==0)
  MaxV=1;
end
% Iteration : Crank Nicolson scheme
for k=1:niter,
% u1 means u^n
% u2 means u^{n+1}
  SM=b*sqrt(a/b)*sqrt(2/dt)*(SM_new+SM_old)/2/E;
  y=L\(M2*u1+SM);
  u2=U\y;
  u1=u2;
  
  Vlt=[Vlt u2(1)*exp(i*Vl*k*dt)];
  Vrt=[Vrt u2(N+2)*exp(i*Vr*k*dt)];
  
  al=alpha(2:k+1);
  SM_old=SM_new;
  SM_new(1)=exp(-i*Vl*(k+1)*dt)*fliplr(Vlt(2:k+1))*al;
  SM_new(N+2)=exp(-i*Vr*(k+1)*dt)*fliplr(Vrt(2:k+1))*al;
  
% Drawing
  if rem(k,25)==0
  figure(10)
  plot(x,abs(u1),'b',x,real(u1),'r',x,imag(u1),'g',x,potential_func(x)/MaxV,'r:'); hold on
  drawnow; pause(0.1)
% $$$         % Exact solution
% $$$         u_exact = sqrt(i/(-4*dt*k+i)).*exp((-i*xnorm.^2-k0*xnorm+k0^2*dt*k)/(-4*dt*k+i));
% $$$         K=[K;k*dt];
% $$$         %ERR=[ERR;max(abs(u1)-abs(u_exact))];
% $$$         unorm=zeros(N+2+N+1,1);
% $$$         unorm(2:2:2*N+3)=(u1(1:N+1)+u1(2:N+2))/2;
% $$$         unorm(1:2:2*N+3)=u1;
% $$$         l2=l2simpson(dx,abs(unorm));
% $$$         l2ex=l2simpson(dx,abs(u_exact));
% $$$         L2NORM=[L2NORM;l2];
% $$$         L2NORMEX=[L2NORMEX;l2ex];
% $$$         u_eccard=unorm-u_exact;
% $$$         ERR=[ERR;l2simpson(dx,abs(u_eccard))/l2ex];      
% $$$         sav=sav+1;
% $$$         %K=[K;k*dt];ERR=[ERR;l2simpson(dx,abs(u_eccard))/l2ex];      
    sav=sav+1;
    pl(sav,:)=u1';
    tl(sav)=k*dt;
    if   data.PlotFlag
      plot(x,abs(u1),x,potential_func(x)/MaxV,'r:','parent',data.axe1);axis(data.axe1,[x0 x1 0 1.1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
	pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
	axis(data.axe2,[x0 x1 0 tfinal]);
	h=colorbar('peer',data.axe2);
	set(h,'fontsize',7);
      end
    end    
  end
end
% $$$   % Plot of the log-error
% $$$   pl2=log10(pl);
% $$$   contourf(x,tl,pl2,[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16]);colorbar;
% $$$   colormap(jet);
if   data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	   'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end
% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end 