function [out]=funcpade2(data)
% Antoine-Besse June 2006
% Computation of the solution of the free Schroedinger equation
% by Crank-Nicolson scheme with finite differences method
% with Absorbing Boundary conditions : discretization of Szeftel

% data is a cell
%  |
%  |--- data.a
%  |--- data.b
%  |--- data.sigma
%  |--- data.xc
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.initDatum
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
savenb=data.savenb;
a=data.a;
b=data.b;
sigma=data.sigma;
xc=data.xc;
x0=data.x0; 
x1=data.x1;
N=data.N;
dx=(x1-x0)/(N+1);
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=tfinal/dt;
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end
% Wave number
k0=data.k0;

%--------------------------------------------------------------
%--------------------------------------------------------------

%
% Computation of convolution terms
%
%Number of elements for the convolution
m=data.m;
COEF=zeros(2*m+2,1);
COEF(1)=m;
COEF(2)=0;
k=0;
for kk=3:m+2
  COEF(kk)=1/cos((2*k+1)*pi/4/m)^2/m;
  k=k+1;
end
k=0;
for kk=m+3:2*m+2
  COEF(kk)=tan((2*k+1)*pi/4/m)^2;
  k=k+1;
end
am=COEF(2:m+2);
dm=COEF(m+3:2*m+2);
sam=sum(am);
Vl=potential_func(x0); % Potential value on xl
Vr=potential_func(x1); % Potential value on xr
padml=i*b*dt*am(2:m+1).*dm./(2*a+i*Vl*dt-i*b*dt*dm);
padmr=i*b*dt*am(2:m+1).*dm./(2*a+i*Vr*dt-i*b*dt*dm);
cofl=sam+sum(padml);
cofr=sam+sum(padmr);
%  cof=(i-dm*dt);


x=(x0:dx:x1)';
gridx = x(2:N+1);
% Initial datum
%u1=initialDatum(x,k0);
u1=eval(data.initDatum);  
% try
%   V = potential_func(data.xc);
%   a = data.a;
%   b = data.b;
%   xc = data.xc;
%   k0 = data.k0;
%   sigma = data.sigma;
%   t = 0;
%   u1=eval(data.exactCpot);
% catch
%    warning('Using initDatum') 
% end
u2=u1;
u1(1)=0;u1(N+2)=0;

%-----------------------------------------------------------
% Matrices assembly
%-----------------------------------------------------------
pot=potential_func(x);
dfract=(sqrt(2)-i*sqrt(2))/2;
e=ones(N+2,1);
e1=(b/dx^2)*e;
e2=(2*i*a/dt-2*b/dx^2)*e-pot;
A=spdiags([e1 e2 e1],[-1 0 1],N+2,N+2);
A(1,1)=0.5/dx;
A(1,2)=-i*cofl;
A(1,3)=-0.5/dx;
A(N+2,N+2)=0.5/dx;
A(N+2,N+1)=-i*cofr;
A(N+2,N)=-0.5/dx;
phil=zeros(m,1);
phir=phil;

SM=zeros(N+2,1);

% LU factorization of matrix A

%[L,U]=lu(A);
[LA,UA,pA,qA] = lu(A,'vector');
sav=1;
% pl : used to store |u^n|
pl=zeros(niter/savenb,N);
pl(1,:)=u1(2:N+1).';
xpl=x(2:N+1);
tl=zeros(niter/savenb,1);
tl(1)=0;

%--------------------------------------------------------------
%--------------------------------------------------------------

cof1phil=(i*a/dt+Vl/2-b*dm/2)./(i*a/dt-Vl/2+b*dm/2);
cof1phir=(i*a/dt+Vr/2-b*dm/2)./(i*a/dt-Vr/2+b*dm/2);
cof2phil=b./(i*a/dt-Vl/2+b*dm/2);
cof2phir=b./(i*a/dt-Vr/2+b*dm/2);
mycofl=2*a*am(2:m+1).*dm./(2*a+i*Vl*dt-i*b*dt*dm);
mycofr=2*a*am(2:m+1).*dm./(2*a+i*Vr*dt-i*b*dt*dm);
cdt=2*i*a/dt;

MaxV=max(potential_func(x));
if (MaxV==0)
  MaxV=1;
end;
% Iterations
for k=1:niter,
% u1 means u^n
% u2 means u^{n+1}
% Resolution of the linear system
  rhs = cdt*u1+SM;
  v(qA,:)= UA\(LA\rhs(pA,:));
%  y=L\(cdt*u1+SM);
%  v=U\y;
  u2=2*v-u1;
% Replacement of the old u^n by u^(n+1)
  u1=u2;
  u1(1)=0;u1(N+2)=0;
  phil=cof1phil.*phil+cof2phil*v(2);
  phir=cof1phir.*phir+cof2phir*v(N+1);
% Computation of the futur R.H.S by convolution
  SM(1)=-i*mycofl.'*phil;
  SM(N+2)=-i*mycofr.'*phir;
  
% Display
  if rem(k,savenb)==0
% $$$       fprintf('Iteration %d\n',k);
% $$$       plot(x,abs(u1));axis([x0 x1 0 1]);drawnow;
    sav=sav+1;
    pl(sav,:)=u1(2:N+1).';
    tl(sav)=k*dt;
    if   data.PlotFlag
      plot(x,abs(u1),x,potential_func(x)/MaxV,'r:','parent',data.axe1);axis(data.axe1,[x0 x1 0 1.1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
        pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
        contourf(xpl,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
        axis(data.axe2,[x0 x1 0 tfinal]);
        h=colorbar('peer',data.axe2);
        set(h,'fontsize',7);
      end
    end
  end
end
% $$$   figure(2);
% $$$   mesh(x,tl,pl);view(160,60);
if   data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(xpl,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	   'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end
% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end 