function [out]=funcpmlfem(data)
% Computation of the solution of the free Schroedinger equation
% by Crank-Nicolson scheme with finite differences method
% with Absorbing Boundary conditions : PML
% data is a cell
%  |
%  |--- data.a
%  |--- data.b
%  |--- data.sigma
%  |--- data.xc
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.initDatum
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
% Setting default values
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
a=data.a;
b=data.b;
savenb=data.savenb;
sigma=data.sigma;
xc=data.xc;
% Wave number
k0=data.k0;

x0 = data.x0;
x1 = data.x1;
nopt= data.N;
x = linspace(x0,x1,nopt).'; % points of computational domain
gridx = x;
dx = x(2)-x(1);
e = [[1:nopt-1].' [2:nopt].']; % 1D grid
Para = 0;

tfinal = data.tfinal;
dt = data.dt;
niter=tfinal/dt;

% Initial datum
u_n_D = eval(data.initDatum);  
% try
%   V = potential_func(data.xc);
%   a = data.a;
%   b = data.b;
%   xc = data.xc;
%   k0 = data.k0;
%   sigma = data.sigma;
%   t = 0;
%   u_n_D = eval(data.exactCpot); 
% %  u_n_D([1 end])
% catch
%    warning('Using initDatum') 
% end
% 
sigma  = 1;
sigmaL = sigma;              % damping parameter
sigmaR = sigma;
epsilon = 1e-8;              % user given tolerance
lambdamin = 0.1;             % the smallest wavelength that is present in the problem
kmin = 0.1;                  % the smallest wavenumber

npw = lambdamin/dx;          % points per wavelength

% The PML is choose adaptively following LIN's suggestion, 
% cf. Domain Decomposition paper.
xi = adaptivepml(epsilon,sigma,dx,kmin,npw);

xPML_L = x0-fliplr(xi).';
xPML_R = x1+xi.';

noptL = length(xPML_L);
noptR = length(xPML_R);

P_INT  = sparse([1:nopt]+noptL-1,[1:nopt],ones(1,nopt),nopt+noptR+noptL-2,nopt);
P_LPML = sparse([1:noptL],[1:noptL],ones(1,noptL),nopt+noptR+noptL-2,noptL);
P_RPML = sparse([1:noptR]+nopt+noptL-2,[1:noptR],ones(1,noptR),nopt+noptR+noptL-2,noptR);

ePML_L = [[1:noptL-1].' [2:noptL].'];
ePML_R = [[1:noptR-1].' [2:noptR].'];

[A_PD,M_PD,V_PD]    = assemble_AM(x,e,Para); %M mass matrix; A stiffness matrix (-Laplace);
                                    %V potential 
[A_LL,M_LL,V_LL] = assemble_AM(xPML_L,ePML_L,Para);
[A_RL,M_RL,V_RL] = assemble_AM(xPML_R,ePML_R,Para);


AG = P_INT*A_PD*P_INT.' + (1/(1+i*sigmaL))*P_LPML*A_LL*P_LPML.'  + (1/(1+i*sigmaR))* P_RPML*A_RL*P_RPML.';
MG = P_INT*M_PD*P_INT.' + (1+i*sigmaL)*P_LPML*M_LL*P_LPML.'  + (1+i*sigmaR)* P_RPML*M_RL*P_RPML.';
VG = P_INT*V_PD*P_INT.' + (1+i*sigmaL)*P_LPML*V_LL*P_LPML.'  + (1+i*sigmaR)* P_RPML*V_RL*P_RPML.';

if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end

%--------------------------------------------------------------
%--------------------------------------------------------------

AA = i*a/dt*MG - 1/2*(b*AG + VG);
BB = i*a/dt*MG + 1/2*(b*AG + VG);

%[L,U] = lu(AA);
[LA,UA,pA,qA] = lu(AA,'vector');
% % Initial datum
% u_n_D = eval(data.initDatum);  
% try
%   V = potential_func(data.xc);
%   a = data.a;
%   b = data.b;
%   xc = data.xc;
%   k0 = data.k0;
%   sigma = data.sigma;
%   t = 0;
%   u_n_D = eval(data.exactCpot); 
% %  u_n_D([1 end])
% catch
%    warning('Using initDatum') 
% end
u = P_INT * u_n_D;

sav=1;
% pl : used to store |u^n|
pl=zeros(niter/savenb,nopt);
pl2=pl;
pl(1,:)=u_n_D.';
tl=zeros(niter/savenb,1);
tl(1)=0;


%--------------------------------------------------------------
%--------------------------------------------------------------
MaxV=max(potential_func(x));
if (MaxV==0)
  MaxV=1;
end
% Iterations
for k=1:niter,
  
%   u_n1 = U\(L\(BB*u_n));
%   u_n = u_n1;
  rhs = BB*u;
  u(qA,:)= UA\(LA\rhs(pA,:));
  
% Display
  if rem(k,savenb)==0
%      figure(10)
%      plot(x,abs(P_INT.'*u_n),'k:',x,real(P_INT.'*u_n),'m:',x,imag(P_INT.'*u_n),'c:',x,potential_func(x)/MaxV,'r:'); hold on
%      drawnow; pause(0.1)
     sav=sav+1;
    pl(sav,:)=(P_INT.'*u).';
    tl(sav)=k*dt;
    if   data.PlotFlag   
      plot(x,abs(P_INT.'*u),x,potential_func(x)/MaxV,'r:','parent',data.axe1);axis(data.axe1,[x0 x1 0 1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
	pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
	contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16]);
	axis(data.axe2,[x0-1 x1+1 0 tfinal]);
	h=colorbar('peer',data.axe2);
	set(h,'fontsize',7);
      end
    end
  end
end

% $$$   figure(2);
% $$$   mesh(x,tl,pl);view(160,60);
if data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	   'parent',data.axe2);
  axis(data.axe2,[x0-1 x1+1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
end

% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end

function [xi] = adaptivepml(epsilon,sigma,dx,kmin,npw)

xi(1) = dx;
N = 1;
mle = -log10(epsilon);
dpml = dx;

while mle/(xi(N)*sigma) > kmin
  dpml = min(2*pi*sigma*xi(N)/mle/npw,1.6*dpml);
  xi(N+1) = xi(N) + max(dx,dpml);
  if xi(N+1) > 1/epsilon
    return
  else
    N = N+1;
  end
end

xi = [0 xi];
