function [out]=funcszeftel(data)
% Antoine-Besse June 2006
% Computation of the solution of the free Schroedinger equation
% by Crank-Nicolson scheme with finite differences method
% with Absorbing Boundary conditions : discretization of Szeftel

% data is a cell
%  |
%  |--- data.x0
%  |--- data.x1
%  |--- data.N
%  |--- data.tfinal
%  |--- data.dt
%  |--- data.k0  
%  |--- data.m
%  |--- data.axe1  : where to plot |u|
%  |--- data.axe2  : where to plot contour of log10|u|
%  |--- data.txtiter    : handle on textiter
%  |--- data.txtnbiter  : handle on textnbiter

%  clear all;
% Setting default values
if ~isfield(data,'PlotFlag')
  data.PlotFlag = logical(1);
end
if ~isfield(data,'ReturnFlag')
  data.ReturnFlag = 1;
end

% Data
% Computational domain : [x0,x1]
% N+2 points : extremity set on points 1 and N+2
x0=data.x0; 
x1=data.x1;
N=data.N;
dx=(x1-x0)/(N+1);
% Final time of computation
% dt : time step
% niter : number of iterations
tfinal=data.tfinal;
dt=data.dt;
niter=tfinal/dt;
if data.PlotFlag
  set(data.txtnbiter,'String',num2str(niter));  
end
% Wave number
k0=data.k0;

%--------------------------------------------------------------
%--------------------------------------------------------------

%
% Computation of convolution terms
%
%Number of elements for the convolution
m=3;
COEF=[3;0.7269284;2.142767;5.742223;46.58032;6.906263;65.82243;1124.376];
am=COEF(2:m+2);
dm=COEF(m+3:2*m+2);
sam=sum(am);
padm=am(2:m+1).*dm;
cof=(1-dm*dt);

u1=zeros(N+2,1);
u2=u1;
x=(x0:dx:x1)';

% Initial datum
u1=initialDatum(x,k0);
u2=u1;

%-----------------------------------------------------------
% Matrices assembly
%-----------------------------------------------------------
dfract=(sqrt(2)-i*sqrt(2))/2;
e=ones(N+2,1);
e1=(0.5/dx^2)*e;
e2=(i/dt-1/dx^2)*e;
e3=(i/dt+1/dx^2)*e;
A=spdiags([e1 e2 e1],[-1 0 1],N+2,N+2);
A(1,1)=1/dx+dfract*sam;
A(1,2)=-1/dx;
A(N+2,N+2)=1/dx+dfract*sam;
A(N+2,N+1)=-1/dx;
B=spdiags([-e1 e3 -e1],[-1 0 1],N+2,N+2);
B(1,:)=0;
B(N+2,:)=0;
phil=zeros(m,1);
phir=phil;
phil=cof.*phil+dt*u1(1);
phir=cof.*phir+dt*u1(N+2);

SM=zeros(N+2,1);
SM(1)=dfract*padm'*phil;
SM(N+2)=dfract*padm'*phir;

% LU factorization of matrix A

%[L,U]=lu(A);
[LA,UA,pA,qA] = lu(A,'vector');

sav=1;
% pl : used to store |u^n|
pl=zeros(niter/25,N+2);
pl(1,:)=u1.'; gridx = x;
tl=zeros(niter/25,1);
tl(1)=0;

%--------------------------------------------------------------
%--------------------------------------------------------------

% Iterations
for k=1:niter,
% u1 means u^n
% u2 means u^{n+1}
% Resolution of the linear system
  rhs =B*u1+SM;
  u2(qA,:)= UA\(LA\rhs(pA,:));
%   y=L\(B*u1+SM);
%   u2=U\y;
% Replacement of the old u^n by u^(n+1)
  u1=u2;
  phil=cof.*phil+dt*u1(1);
  phir=cof.*phir+dt*u1(N+2);
% Computation of the futur R.H.S by convolution
  SM(1)=dfract*padm'*phil;
  SM(N+2)=dfract*padm'*phir;
  
% Display
  if rem(k,25)==0
% $$$       fprintf('Iteration %d\n',k);
% $$$       plot(x,abs(u1));axis([x0 x1 0 1]);drawnow;
    sav=sav+1;
    pl(sav,:)=u1.';
    tl(sav)=k*dt;
    if data.PlotFlag
      plot(x,abs(u1),'parent',data.axe1);axis(data.axe1,[x0 x1 0 1]);drawnow;
      set(data.txtiter,'String',num2str(k));
      if rem(sav,20)==0
        pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
        contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
		 'parent',data.axe2);
        axis(data.axe2,[x0 x1 0 tfinal]);
        h=colorbar('peer',data.axe2);
        set(h,'fontsize',7);
      end
    end
  end
end
% $$$   figure(2);
% $$$   mesh(x,tl,pl);view(160,60);
if data.PlotFlag
  pl2(1:sav,:)=log10(abs(pl(1:sav,:)));
  contourf(x,tl(1:sav),pl2(1:sav,:),[0,-0.5,-1,-2,-3,-3.5,-4,-4.5,-5,-6,-16],...
	   'parent',data.axe2);
  axis(data.axe2,[x0 x1 0 tfinal]);
  h=colorbar('peer',data.axe2);
  set(h,'fontsize',7);
  end

% Output
if data.ReturnFlag == 1
  out = pl;
elseif data.ReturnFlag == 2
  out.pl = pl;
  out.tl = tl;
  out.gridx = gridx;
else
  error(sprintf('Unknown ReturnFlag %g',ReturnFlag))
end